Attribute VB_Name = "mILDuplicate"
'*****************************************************************
' Module to duplicate imagelist from remote process
' using remote API call.
' Written by Arkadiy Olovyannikov (ark@msun.ru)
' Copyright 2005 by Arkadiy Olovyannikov
'
' This software is FREEWARE. You may use it as you see fit for
' your own projects but you may not re-sell the original or the
' source code.
'
' No warranty express or implied, is given as to the use of this
' program. Use at your own risk.
'*****************************************************************
Option Explicit

Private Type PictDesc
    cbSizeofStruct As Long
    PicType As Long
    hImage As Long
    xExt As Long
    yExt As Long
End Type

Private Type Guid
    Data1 As Long
    Data2 As Integer
    Data3 As Integer
    Data4(0 To 7) As Byte
End Type

Private Declare Function OleCreatePictureIndirect Lib "OLEPRO32.DLL" (lpPictDesc As PictDesc, riid As Guid, ByVal fPictureOwnsHandle As Long, ipic As IPicture) As Long
Private Declare Function DestroyIcon Lib "user32" (ByVal hIcon As Long) As Long
Private Const ILD_TRANSPARENT = 1
Private Const ILD_PRESERVEALPHA = &H1000
Private Const CLR_NONE = -1

'********************************************************************************
'Main function to duplicate remote image list and fill VB ImageList with
'appropriate icons.
'********************************************************************************
Public Function IL_Duplicate(ByVal hProcess As Long, ByVal hIml As Long, il As ImageList) As Long
   If (hIml = 0) Or (hProcess = 0) Or (il Is Nothing) Then Exit Function
   
   Dim i As Long, nCount As Long, bkColor As Long, maskColor As Long, hIcon As Long, ret As Long
   Dim dt() As API_DATA
   Dim hLib As Long, hLibUser As Long, fnAddr As Long, fnDestroy As Long
   Dim bNeedUnload As Boolean
   Dim sPic As StdPicture

   il.ListImages.Clear
'Get comctl32 instance (load if missing)
   hLib = GetModuleHandleRemote(hProcess, "comctl32")
   hLibUser = GetModuleHandleRemote(hProcess, "user32")
   If hLibUser = 0 Then Exit Function
   If hLib = 0 Then
      hLib = LoadLibraryRemote(hProcess, "comctl32")
      bNeedUnload = True
   End If
   If hLib = 0 Then Exit Function
   
'Get image count
   fnAddr = GetProcAddressRemote(hProcess, hLib, "ImageList_GetImageCount")
   If fnAddr = 0 Then GoTo CleanUp
   
   ReDim dt(2)
   For i = 0 To 2 'Set default parameters
       dt(i).argType = arg_Value
       dt(i).dwDataLength = 4
   Next i
   dt(0).lpData = hIml
   nCount = CallFunctionRemote(hProcess, fnAddr, 1, dt, 5000)
   If nCount = 0 Then GoTo CleanUp

'Get ImageList back color
   fnAddr = GetProcAddressRemote(hProcess, hLib, "ImageList_GetBkColor")
   bkColor = vbWhite
   If fnAddr Then
      bkColor = CallFunctionRemote(hProcess, fnAddr, 1, dt, 5000)
      If bkColor <> CLR_NONE Then il.BackColor = bkColor
   End If

'Extract icons from source ImageList one by one and add them to VB ImageList
   fnAddr = GetProcAddressRemote(hProcess, hLib, "ImageList_GetIcon")
   fnDestroy = GetProcAddressRemote(hProcess, hLibUser, "DestroyIcon")
   dt(2).lpData = ILD_TRANSPARENT Or ILD_PRESERVEALPHA
   For i = 0 To nCount - 1
       dt(0).lpData = hIml
       dt(1).lpData = i
       hIcon = CallFunctionRemote(hProcess, fnAddr, 3, dt, 5000)
       Set sPic = IconToPicture(hIcon)
       il.ListImages.Add , , sPic
       Set sPic = Nothing
       dt(0).lpData = hIcon
       CallFunctionRemote hProcess, fnDestroy, 1, dt, 5000
   Next i
   IL_Duplicate = nCount
'Free objects
CleanUp:
   If bNeedUnload Then FreeLibraryRemote hProcess, hLib
End Function

Private Function IconToPicture(ByVal hIcon As Long) As StdPicture
    If hIcon = 0 Then Exit Function
    Dim oNewPic As Picture
    Dim tPicConv As PictDesc
    Dim IGuid As Guid
    With tPicConv
       .cbSizeofStruct = Len(tPicConv)
       .PicType = vbPicTypeIcon
       .hImage = hIcon
    End With
    With IGuid
        .Data1 = &H7BF80980
        .Data2 = &HBF32
        .Data3 = &H101A
        .Data4(0) = &H8B
        .Data4(1) = &HBB
        .Data4(2) = &H0
        .Data4(3) = &HAA
        .Data4(4) = &H0
        .Data4(5) = &H30
        .Data4(6) = &HC
        .Data4(7) = &HAB
    End With
    OleCreatePictureIndirect tPicConv, IGuid, True, oNewPic
    Set IconToPicture = oNewPic
End Function

